@@ -252,7 +252,7 @@ bool AspScanList::Redisplay()
     try
     {
 #endif
-        foreachc(*this, DataLineList)
+        foreachc(*this)
         {
             auto ad = SC(const AspData *, !it);
             if (ad->asp == Unknown_Aspect) continue;
@@ -371,7 +371,7 @@ bool AspScanList::Redisplay()
             DrawTabText(i18n("To"));
             NewLine();
             NewLine();
-            foreachc(voclist, AstroList<Voc>)
+            foreachc(voclist)
             {
                 auto v = SC(const struct Voc *, !it);
                 int o = (ar * v->a)->Orb;
@@ -474,7 +474,7 @@ bool AspScan::DumpAspects()
             if (arq.GetInt("count", 0) > 0)
                 arq.Exec("DELETE FROM AspTbl WHERE Idx = %d", TblIdx);
         }
-        foreachc(*this, DataLineList)
+        foreachc(*this)
         {
             auto ad = SC(const AspData *, !it);
             dat.SetJd(ad->jd);
