@@ -126,13 +126,13 @@ AstroMainWindow::AstroMainWindow()
     QNetworkRequest request;
     request.setUrl(QUrl(QString("https://skylendar.sourceforge.io/Msg.php?Id=%1").arg(Asr->Id)));
 #ifdef __WINNT
-    request.setRawHeader("User-Agent", "Skylendar/4.2.0 (W20)");
+    request.setRawHeader("User-Agent", "Skylendar/4.2.0 (W21)");
 #elif defined(__linux__)
-    request.setRawHeader("User-Agent", "Skylendar/4.2.0 (L20)");
+    request.setRawHeader("User-Agent", "Skylendar/4.2.0 (L21)");
 #elif defined(__APPLE__)
-     request.setRawHeader("User-Agent", "Skylendar/4.2.0 (M20)");
+     request.setRawHeader("User-Agent", "Skylendar/4.2.0 (M21)");
 #else
-     request.setRawHeader("User-Agent", "Skylendar/4.2.0 (?20)");
+     request.setRawHeader("User-Agent", "Skylendar/4.2.0 (?21)");
 #endif
     request.setRawHeader("HTTP_ACCEPT_LANGUAGE", "en");
     QNr = QNam->get(request);
@@ -471,7 +471,7 @@ const AstroString& AstroMainWindow::RenameWindow(AstroString& as)
 {
     int count = 0;
     AstroString buf;
-    foreachc(*this, AstroMainWindow)
+    foreachc(*this)
     if ((!it)->windowTitle() == as)
         count++;
     return count ? Sprintf("%a (%d)", &buf, &as, count) : as;
@@ -938,7 +938,7 @@ void AstroMainWindow::Save()
 
 void AstroMainWindow::DeModif(Index idx, bool data)
 {
-    foreach(*this, AstroMainWindow)
+    foreach(*this)
     {
         AstroWindow& aw = *(*it);
         for (int i = 0; i < NBDATA; i++)
@@ -951,7 +951,7 @@ void AstroMainWindow::PaintPreview(QPrinter* printer)
 	auto aw = ActiveWindow();
 	if (!aw) return;
 	aw->SetDisplayMode(AstroGraphicChart::PreviewMode);
-    QPrinter *orig = aw->Qprint;
+    auto orig = aw->Qprint;
 	aw->Qprint = printer;
 	aw->PreRedraw(true);
     aw->Acb->Redisplay();
@@ -1009,7 +1009,7 @@ void AstroMainWindow::UpdateCharts(Index idx, bool data)
 {
     bool b = Asr->Ctrl;
     Asr->Ctrl = true;
-    foreach(*this, AstroMainWindow)
+    foreach(*this)
     {
         AstroWindow& aw = *(*it);
         if (aw.GetCt() == AspScList || aw.GetCt() == Ephemeris || aw.GetCt() == Directions || aw.GetCt() == Batch)
@@ -1094,7 +1094,7 @@ void AstroMainWindow::DupWindow()
 void AstroMainWindow::DeleteData()
 {
     const AstroData* ad = Aiw->GetData();
-    foreachc(*Ahl, AstroChartList)
+    foreachc(*Ahl)
     {
         if (!Aiw->Aib)
         {
@@ -1113,7 +1113,7 @@ void AstroMainWindow::DeleteData()
 void AstroMainWindow::DeleteRestr()
 {
     const AstroRestrictions* ar = Aiw->GetRestrictions();
-    foreachc(*Ahl, AstroChartList)
+    foreachc(*Ahl)
     {
         if (!Aiw->Aib)
         {
@@ -1223,19 +1223,7 @@ void AstroMainWindow::Image()
 
 void AstroMainWindow::Doc()
 {
-	QDesktopServices::openUrl(QUrl("https://skylendar.sourceforge.io"));
-	// auto t = new QTextBrowser;
-	// t->setWindowTitle(i18n("Online Documentation"));
-	// t->resize(600, 400);
-	// t->setWindowModality(Qt::NonModal);
-	// t->setAttribute(Qt::WA_DeleteOnClose);
-	// t->setSearchPaths(QStringList(SkyPath));
-	// QFile f("http://sourceforge.net/skylendar/index.html");
-	// f.open(QIODevice::ReadOnly | QIODevice::Text);
-	// QString s = f.readAll();
-	// f.close();
-	// t->setHtml(s);
-	// t->show();
+	QDesktopServices::openUrl(QUrl("https://skylendar.sourceforge.io/index.html"));
 }
 
 void AstroMainWindow::Finished(QNetworkReply *reply)
@@ -1263,7 +1251,7 @@ void AstroMainWindow::customEvent(QEvent* c)
 {
     auto ae = DC(const AstroEvent*, c);
 		if (ae->type() != 1234) return;
-        foreach(*this, AstroMainWindow)
+        foreach(*this)
             if (DC(ParamListBase*, (*it)->Acb) == ae->Plb)
             {
                 ae->Plb->Finish();
