@@ -688,7 +688,7 @@ void WriteSKIF(const AstroString& file, AstroData* ad)
         }
     }
     else
-        foreachc(*Adl, AstroDataList)
+        foreachc(*Adl)
     {
         if ((!it)->Idx != 0)
         {
