@@ -65,7 +65,7 @@
 class MyScrollArea: public QScrollArea
 {
 public:
-    MyScrollArea(AstroWindow* asw, QWidget* parent = 0): QScrollArea(parent) {
+    MyScrollArea(AstroWindow* asw, QWidget* parent = nullptr): QScrollArea(parent) {
         Asw = asw;
     }
 
@@ -425,7 +425,7 @@ AstroIconsWindow& AstroWindow::Aiw() const
 void AstroWindow::ManageTypes(int i)
 {
     auto cti = (enum ChartType)i;
-    auto plb = DC(ParamListBase*, Acb);
+    //auto plb = DC(ParamListBase*, Acb);
     Acb->Clean();
     if (cti < StdWheels || cti >= Unknown_ChartType)
         return;
@@ -547,7 +547,7 @@ int AstroWindow::GetRing() const
     return Amw->Ring -1;
 }
 
-AstroGraphicChart::AstroGraphicChart(int width, int height): AstroGraphics(0, 0, 0, width, height), AstroChart()
+AstroGraphicChart::AstroGraphicChart(int width, int height): AstroGraphics(nullptr, 0, 0, width, height), AstroChart()
 {
     Cbp->Ag = Base = this;
     Cpr = Cbp;
@@ -620,17 +620,15 @@ AstroPrintPage::~AstroPrintPage()
 
 void AstroPrintPage::NewPage()
 {
-    if(Ww == 0.0)
+    if(Agc->Ww == 0.0)
     {
-        Ww = float(Agc->W) / Agc->GetWidth();
-        Ww -= Ww / 50.0;
-        Height = int(float(Agc->H) / Ww);
-        Agc->Qp1.scale(Ww, Ww);
+        auto Gcb = SC(GraphicChartBase*, Acb);
+        if(Gcb) Gcb->GetWH(Agc->W, Agc->H);
+        Height = Agc->H;
     }
-    if(Page == 1) Header();
     Footer(Acb->IfPage);
     Agc->Qp->end();
-    Agc->Qp1.drawPicture(0, 0, *Agc->QPictPrint);
+    Agc->Qp1.drawPicture(Agc->Offset, 0, *Agc->QPictPrint);
     //Agc->QPictPrint->play(Agc->Qp);
     Agc->Qprint->newPage();
     delete Agc->QPictPrint;
@@ -646,11 +644,12 @@ void AstroPrintPage::Header()
     int w = Agc->GetWidth(), h;
     enum AstroGraphics::SizeChars sc = Agc->GetSc();
     Agc->FontSize(AstroGraphics::Tiny);
-    Agc->TextMetrics("O", 0, &h);
+    Agc->TextMetrics("O", nullptr, &h);
     h /=2;
     AstroString as, l = PACKAGE;
     l += " ";
     l += VERSION;
+    int xi = Agc->Xi, yi = Agc->Yi;
     Agc->Move(0, h);
     Agc->JustifText(Now->GetDate(as), AstroGraphics::Left, w);
     Agc->Move(0, h);
@@ -658,21 +657,24 @@ void AstroPrintPage::Header()
     Agc->Move(0, h);
     Agc->JustifText(l, AstroGraphics::Right, w);
     Agc->FontSize(sc);
+    Agc->Xi = xi;
+    Agc->Yi = yi;
 }
 
 void AstroPrintPage::Footer(bool ifpage)
 {
-    int w = Agc->GetWidth(), h;
+    int w = Agc->GetWidth(), h, h1;
     enum AstroGraphics::SizeChars sc = Agc->GetSc();
     Agc->FontSize(AstroGraphics::Tiny);
-    Agc->TextMetrics("O", 0, &h);
+    Agc->TextMetrics("O", nullptr, &h);
+    h1 = Height - h;
     AstroString r;
     Sprintf(i18n("Page %d"), &r, Page);
-    Agc->Move(0, Height);
+    Agc->Move(0, h1);
     if (ifpage) Agc->JustifText(Page % 2 ? "" : r, AstroGraphics::Left, w);
-    Agc->Move(0, Height);
+    Agc->Move(0, h1);
     Agc->JustifText(Asr->Astrologer, AstroGraphics::Center, w);
-    Agc->Move(0, Height);
+    Agc->Move(0, h1);
     if (ifpage) Agc->JustifText(Page % 2 ? r : "", AstroGraphics::Right, w);
     Agc->FontSize(sc);
 }
@@ -741,6 +743,7 @@ void AstroGraphicChart::PreRedraw(bool b)
         delete QPict;
         QPict = new QPicture;
         Qp->begin(QPict);
+        //CSC(QPen&, Qp->pen()).setWidth(BOLDLINE * 30);
         break;
     case PrintMode:
     case PreviewMode:
@@ -748,19 +751,34 @@ void AstroGraphicChart::PreRedraw(bool b)
         Qp->begin(QPictPrint);
         Asr->IfPrint = Acb->IfPage = true;
         GetTitle(as);
-        App = new AstroPrintPage(as, Acb, this);
+        App = mu<AstroPrintPage>(as, Acb, this);
         PrinterSize(W, H, mx, my);
         auto Gcb = SC(GraphicChartBase*, Acb);
-        if(Gcb) Gcb->GetH = [this]()
+        if(Gcb) Gcb->GetWH = [&](int& w, int& h)
         {
-            float ww = float(W) / GetWidth();
+            int w1 = W, f = W * h / w;
+            if(H < f)
+                w1 = W * H / f;
+//            if(!Offset)
+//                Offset = (((w / PRINTZOOMFACT) - w1) / 2);
+            float ww =float(w1) / GetWidth();
+            W = w1;
             ww -= ww / 50.0;
-            return  int(float(H) / ww);
+            h = float(H) / ww;
+            Ww = ww;
+            Qp1.scale(ww, ww);
+            App->SetHeight(h);
+            if(!IsHeader)
+            {
+                App->Header();
+                IsHeader = true;
+            }
+            return Qprint->pageLayout().orientation() == QPageLayout::Landscape;
         };
+
         Resize(GetWidth() * PRINTZOOMFACT, GetHeight() * PRINTZOOMFACT);
-        auto pn = CSC(QPen&, Qp->pen());
-        pn.setWidth(BOLDLINE);
         Qp1.begin(Qprint);
+        CSC(QPen&, Qp->pen()).setWidth(BOLDLINE);
         break;
     }
     if (Asr->Antialias)
@@ -785,25 +803,24 @@ void AstroGraphicChart::PostRedraw()
         break;
     case PrintMode:
     case PreviewMode:
-        if(App->Ww == 0.0)
+        if(Ww == 0.0)
         {
-            float ww = float(W) / GetWidth();
-            ww -= ww / 50.0;
-            Qp1.scale(ww, ww);
-            int hh = int(float(H) / ww);
-            App->SetHeight(hh);
+            auto Gcb = SC(GraphicChartBase*, Acb);
+            if(Gcb)
+                Gcb->GetWH(W, H);
         }
         App->Footer(true);
         Qp->end();
-        Qp1.drawPicture(0, 0, *QPictPrint);
+        Qp1.drawPicture(Offset, 0, *QPictPrint);
         Qp1.end();
         delete QPictPrint;
         P;
         Resize(OrigWidth, OrigHeight);
-        delete App;
         Asr->IfPrint = false;
-        GraphicChartBase *Gcb = SC(GraphicChartBase*, Acb);
-        if(Gcb) Gcb->GetH = nullptr;
+        auto Gcb = SC(GraphicChartBase*, Acb);
+        if(Gcb) Gcb->GetWH = nullptr;
+        Offset = 0;
+        IsHeader = false;
         break;
     }
     Acb->NoSpot = false;
@@ -1087,7 +1104,7 @@ void AstroWindow::mousePressEvent(QMouseEvent *e)
                 ;
             };
         }
-    default:
+    default: break
         ;
     }
 }
@@ -1354,7 +1371,7 @@ void AstroWindow::SetRate(int r)
 //static const char* const fn[] = { "exasp.png", "dirch.png", "signch.png", "rising.png", "eclipse.png", "ephemeris.png", "pdir.png", "cpdir.png", "occult.png" };
 AstroIconsList::AstroIconsList()
 {
-    int i = 0, j = 0;
+    int i = 0;
     AstroIcon* ai;
     AstroString name, file;
     AstroRequest Arq;
@@ -1386,7 +1403,7 @@ AstroIconsList::AstroIconsList()
 const AstroString& AstroIconsList::GetName(unsigned int idx)
 {
     Index i = 0;
-    foreachc(*this, AstroIconsList)
+    foreachc(*this)
     if (i == idx)
         return (!it)->Name;
     AstroIconsList::Const_Iterator it(*this);
@@ -1445,7 +1462,8 @@ AstroIconsWindow::AstroIconsWindow(AstroMainWindow *amw) : QTreeWidget(amw->cent
     AA1(RestPm, i18n("Save"), 2);
     AA1(RestPm, i18n("Delete"), 3);
     AA1(RestPm, i18n("Duplicate"), 4);
-    //setStyleSheet("background-image: url(:/pics/background.png)");
+    const QColor& c = palette().color(QPalette::Window);
+    setStyleSheet(QString("background: qlineargradient(x1: 0, y1: 0, x2: 1, y2: 0, stop: 0 %1, stop: 1 %2);").arg(c.name()).arg(c.lighter(170).name()));
     connect(this, SIGNAL(itemClicked(QTreeWidgetItem*, int)), SLOT(Clicked(QTreeWidgetItem*, int)));
     connect(this, SIGNAL(itemPressed(QTreeWidgetItem*, int)), SLOT(Pressed(QTreeWidgetItem*, int)));
     connect(this, SIGNAL(itemDoubleClicked(QTreeWidgetItem*, int)), SLOT(DoubleClicked(QTreeWidgetItem*, int)));
@@ -1455,9 +1473,9 @@ AstroIconsWindow::AstroIconsWindow(AstroMainWindow *amw) : QTreeWidget(amw->cent
     for (int i = Show_Hide; i <= Last_SubType; i++)
         //if (i != Direction)
         (void)new AstroIconButton(this, i);
-    foreachc(*Adl, AstroDataList)
+    foreachc(*Adl)
     (void)new AstroIconButton(this, !it);
-    foreachc(*Arl, AstroRestrictionsList)
+    foreachc(*Arl)
     (void)new AstroIconButton(this, !it);
     for (int i = StdWheels; i <= Last_Type; i++)
         (void)new AstroIconButton(this, i, TypeButton);
@@ -1670,7 +1688,7 @@ void AstroIconsWindow::mouseMoveEvent(QMouseEvent * e)
 void AstroIconsWindow::Pressed(QTreeWidgetItem* item, int c)
 {
     auto aib = SC(AstroIconButton*, item);
-    Aib = (item != Data && item != SubTypes) ? aib : 0;
+    Aib = (item != Data && item != SubTypes) ? aib : nullptr;
     switch (QApplication::mouseButtons())
     {
     case Qt::LeftButton:
@@ -1754,33 +1772,25 @@ QTreeWidgetItem* AstroIconsWindow::AstroIconButton::GetParent(const AstroIconsWi
     {
     case SubTypeButton:
         return aiw.SubTypes;
-        break;
     case TypeButton:
         return aiw.Types;
-        break;
     case MajorButton:
         return aiw.Major;
-        break;
     case SwordButton:
         return aiw.Sword;
-        break;
     case CoinButton:
         return aiw.Coin;
-        break;
     case StickButton:
         return aiw.Stick;
-        break;
     case CupButton:
         return aiw.Cup;
-        break;
     default:
-        return 0;
+        return nullptr;
     }
 }
 
 AstroIconsWindow::AstroIconButton::AstroIconButton(AstroIconsWindow* parent, int index, ButtonType bt) : QTreeWidgetItem(GetParent(*parent, bt))
 {
-    extern const char** RomanNumbers;
     Idx = index;
     Bt = bt;
     switch (Bt)
@@ -1797,7 +1807,7 @@ AstroIconsWindow::AstroIconButton::AstroIconButton(AstroIconsWindow* parent, int
     case MajorButton:
     {
         AstroString as;
-        if (index != 0 && index != 22) as = RomanNumbers[index] + AstroString(" ");
+        if (index != 0 && index != 22) as = AstroResources::RomanNumbers[index] + AstroString(" ");
         setText(0, as + *Asr->MajTarots[index]);
         break;
     }
