@@ -1,7 +1,7 @@
 --
 -- Installskydb.sql
 --
--- Copyright Christophe Gros 2017
+-- Copyright Christophe Gros 2018
 -- 
 -- The present script is released under the General Public Licence
 --
@@ -112,12 +112,137 @@ BEGIN
 END;
 $$ LANGUAGE 'plpgsql';
 
---
--- SkylDbInit()
--- The main function for db creation or update
---
+CREATE OR REPLACE FUNCTION Del_Data () RETURNS TRIGGER AS $$
+DECLARE
+	r RECORD;
+	i Integer;
+BEGIN
+	IF OLD.Oidimg <> 0 THEN
+		 RAISE WARNING 'deleted blob %', OLD.OidImg;
+		i := lo_unlink(OLD.OidImg);
+	END IF;
+	SELECT INTO r RefIdx FROM VDV WHERE RefIdx = OLD.Idx AND DataIdx <> RefIdx;
+	IF FOUND THEN
+		RAISE EXCEPTION 'Cannot delete this data set. It is referenced by other data sets'; 
+		RETURN NULL;
+	END IF;
+	DELETE FROM VData WHERE DataIdx = OLD.Idx;
+  RETURN OLD;
+END;
+$$ LANGUAGE 'plpgsql';
 
-CREATE OR REPLACE FUNCTION InitSkyDb() RETURNS Void AS $$
+CREATE OR REPLACE FUNCTION IfSign(Real, Integer) RETURNS Boolean AS $$
+DECLARE
+	angle ALIAS FOR $1;
+	sn ALIAS FOR $2;
+	d INTEGER;
+BEGIN
+	d := FLOOR(angle / 30.0) + 1;
+	IF d = sn THEN
+		RETURN TRUE;
+	ELSE
+		RETURN FALSE;
+	END IF;
+END;
+$$ LANGUAGE 'plpgsql' IMMUTABLE;
+
+CREATE OR REPLACE FUNCTION TObj(Int4, VARCHAR, Integer) RETURNS Boolean AS $$
+DECLARE
+	r RECORD;
+	o ALIAS FOR $2; 
+BEGIN
+	SELECT INTO r DataIdx, Lon FROM Obj WHERE DataIdx = $1 AND Id ~* o AND IfSign(Lon, $3);
+	IF FOUND THEN
+		RETURN TRUE;
+	ELSE
+		RETURN FALSE;
+	END IF;
+END;
+$$ LANGUAGE 'plpgsql' STABLE;
+ 
+CREATE OR REPLACE FUNCTION IfAsp(VARCHAR, VARCHAR, VARCHAR) RETURNS Boolean AS $$
+DECLARE
+	asp VARCHAR;
+	a VARCHAR;
+	b VARCHAR;
+	per CONSTANT VARCHAR := '%';
+	da CONSTANT VARCHAR := '-';
+BEGIN
+	asp := rtrim($1, ' ');
+	a := rtrim($2, ' ');
+	b := rtrim($3, ' ');
+	IF a = '' THEN
+		IF asp LIKE per || b || per THEN
+			RETURN TRUE;
+		END IF;
+	ELSIF b = '' THEN
+		IF asp LIKE per || a || per  THEN
+			RETURN TRUE;
+		END IF;
+	ELSIF asp LIKE (a || da || b) OR asp LIKE (b || da || a) THEN
+			RETURN TRUE;
+	END IF;
+    	RETURN FALSE;
+END;
+$$ LANGUAGE 'plpgsql' IMMUTABLE;
+
+CREATE OR REPLACE FUNCTION TAsp(Int4, VARCHAR, VARCHAR, VARCHAR) RETURNS Boolean AS $$
+DECLARE
+	r RECORD;
+	ai ALIAS FOR $2;
+	o1 ALIAS FOR $3;
+	o2 ALIAS FOR $4; 
+BEGIN
+	SELECT INTO r DataIdx, AspId, Objs FROM Asp WHERE DataIdx = $1 AND AspId ~* ai AND IfAsp(Objs, o1, o2);
+	IF FOUND THEN
+		RETURN TRUE;
+	ELSE
+		RETURN FALSE;
+	END IF;
+END;
+$$ LANGUAGE 'plpgsql' STABLE;
+
+CREATE OR REPLACE FUNCTION TRetr(Int4, VARCHAR) RETURNS Boolean AS $$
+DECLARE
+	ai ALIAS FOR $2;
+	r RECORD;
+BEGIN
+	SELECT INTO r Id, Retro FROM Obj WHERE DataIdx = $1 AND Id ~* ai;
+	IF FOUND THEN 
+		RETURN r.Retro;
+	END IF;
+	RETURN FALSE;
+END;
+$$ LANGUAGE 'plpgsql' STABLE;
+
+CREATE OR REPLACE FUNCTION THouse(Int4, VARCHAR, Integer) RETURNS Boolean AS $$
+DECLARE
+	ai ALIAS FOR $2;
+	r RECORD;
+BEGIN
+	SELECT INTO r InHouse FROM Obj WHERE DataIdx = $1 AND Id ~* ai;
+	IF FOUND THEN
+		IF r.InHouse = $3 THEN
+			RETURN TRUE;
+		END IF;
+	END IF;
+	RETURN FALSE;
+END;
+$$ LANGUAGE 'plpgsql' STABLE;
+
+CREATE OR REPLACE FUNCTION IfGroup() RETURNS Void AS $$
+DECLARE
+    r RECORD;
+BEGIN
+    SELECT INTO r Rolname FROM Pg_Authid WHERE Rolname = 'skyusers';
+    IF NOT FOUND THEN
+      CREATE GROUP SkyUsers;
+    END IF;
+END;
+$$ LANGUAGE 'plpgsql';
+
+DO
+$$
 DECLARE
 	r RECORD;
 	n VARCHAR;
@@ -737,9 +862,11 @@ BEGIN
 ---
 			RAISE WARNING 'Level 4.2';
 			SELECT * INTO r FROM pg_catalog.pg_type WHERE typname = 'aspkind';
+			IF NOT FOUND THEN
+				 SELECT * INTO r FROM pg_catalog.pg_type WHERE typname = 'aspkind';
 			IF NOT FOUND THEN
 				 CREATE TYPE AspKind AS ENUM ('ExAsp', 'ChangeDir', 'ChangeSign', 'Ingress', 'Egress', 'Batch');
-			ALTER TABLE AspTbl ALTER AspKind TYPE AspKind USING
+				 ALTER TABLE AspTbl ALTER AspKind TYPE AspKind USING
 					CASE
 								WHEN AspKind = 'X' THEN 'ExAsp'::AspKind
 								WHEN AspKind = 'D' THEN 'ChangeDir'::AspKind
@@ -751,145 +878,26 @@ BEGIN
 		 		 CREATE SEQUENCE ExtraObjs_Seq;
 		 		 ALTER TABLE ExtraObjs ADD Oid Integer DEFAULT nextval('extraobjs_seq'::regclass);
 			END IF;
+		END IF;
+----
+-- For sk 4.2.1
+----
+				RAISE WARNING 'Level 4.2.1';
+				
+				IF NOT isField('DataIdx', 'AspTbl') THEN
+					ALTER TABLE AspTbl ADD DataIdx Integer;
+				END IF;
+				IF NOT IsField('TruePos', 'Restrictions') THEN
+					 ALTER TABLE Restrictions ADD TruePos Boolean DEFAULT FALSE;
+				END IF;
 
 --- Last step
 ---
 			PERFORM IfGroup();
-END;
-$$ LANGUAGE 'plpgsql';
+END$$;
 
-CREATE OR REPLACE FUNCTION Del_Data () RETURNS TRIGGER AS $$
-DECLARE
-	r RECORD;
-	i Integer;
-BEGIN
-	IF OLD.Oidimg <> 0 THEN
-		 RAISE WARNING 'deleted blob %', OLD.OidImg;
-		i := lo_unlink(OLD.OidImg);
-	END IF;
-	SELECT INTO r RefIdx FROM VDV WHERE RefIdx = OLD.Idx AND DataIdx <> RefIdx;
-	IF FOUND THEN
-		RAISE EXCEPTION 'Cannot delete this data set. It is referenced by other data sets'; 
-		RETURN NULL;
-	END IF;
-	DELETE FROM VData WHERE DataIdx = OLD.Idx;
-  RETURN OLD;
-END;
-$$ LANGUAGE 'plpgsql';
-
-CREATE OR REPLACE FUNCTION IfSign(Real, Integer) RETURNS Boolean AS $$
-DECLARE
-	angle ALIAS FOR $1;
-	sn ALIAS FOR $2;
-	d INTEGER;
-BEGIN
-	d := FLOOR(angle / 30.0) + 1;
-	IF d = sn THEN
-		RETURN TRUE;
-	ELSE
-		RETURN FALSE;
-	END IF;
-END;
-$$ LANGUAGE 'plpgsql' IMMUTABLE;
-
-CREATE OR REPLACE FUNCTION TObj(Int4, VARCHAR, Integer) RETURNS Boolean AS $$
-DECLARE
-	r RECORD;
-	o ALIAS FOR $2; 
-BEGIN
-	SELECT INTO r DataIdx, Lon FROM Obj WHERE DataIdx = $1 AND Id ~* o AND IfSign(Lon, $3);
-	IF FOUND THEN
-		RETURN TRUE;
-	ELSE
-		RETURN FALSE;
-	END IF;
-END;
-$$ LANGUAGE 'plpgsql' STABLE;
- 
-CREATE OR REPLACE FUNCTION IfAsp(VARCHAR, VARCHAR, VARCHAR) RETURNS Boolean AS $$
-DECLARE
-	asp VARCHAR;
-	a VARCHAR;
-	b VARCHAR;
-	per CONSTANT VARCHAR := '%';
-	da CONSTANT VARCHAR := '-';
-BEGIN
-	asp := rtrim($1, ' ');
-	a := rtrim($2, ' ');
-	b := rtrim($3, ' ');
-	IF a = '' THEN
-		IF asp LIKE per || b || per THEN
-			RETURN TRUE;
-		END IF;
-	ELSIF b = '' THEN
-		IF asp LIKE per || a || per  THEN
-			RETURN TRUE;
-		END IF;
-	ELSIF asp LIKE (a || da || b) OR asp LIKE (b || da || a) THEN
-			RETURN TRUE;
-	END IF;
-    	RETURN FALSE;
-END;
-$$ LANGUAGE 'plpgsql' IMMUTABLE;
-
-CREATE OR REPLACE FUNCTION TAsp(Int4, VARCHAR, VARCHAR, VARCHAR) RETURNS Boolean AS $$
-DECLARE
-	r RECORD;
-	ai ALIAS FOR $2;
-	o1 ALIAS FOR $3;
-	o2 ALIAS FOR $4; 
-BEGIN
-	SELECT INTO r DataIdx, AspId, Objs FROM Asp WHERE DataIdx = $1 AND AspId ~* ai AND IfAsp(Objs, o1, o2);
-	IF FOUND THEN
-		RETURN TRUE;
-	ELSE
-		RETURN FALSE;
-	END IF;
-END;
-$$ LANGUAGE 'plpgsql' STABLE;
-
-CREATE OR REPLACE FUNCTION TRetr(Int4, VARCHAR) RETURNS Boolean AS $$
-DECLARE
-	ai ALIAS FOR $2;
-	r RECORD;
-BEGIN
-	SELECT INTO r Id, Retro FROM Obj WHERE DataIdx = $1 AND Id ~* ai;
-	IF FOUND THEN 
-		RETURN r.Retro;
-	END IF;
-	RETURN FALSE;
-END;
-$$ LANGUAGE 'plpgsql' STABLE;
-
-CREATE OR REPLACE FUNCTION THouse(Int4, VARCHAR, Integer) RETURNS Boolean AS $$
-DECLARE
-	ai ALIAS FOR $2;
-	r RECORD;
-BEGIN
-	SELECT INTO r InHouse FROM Obj WHERE DataIdx = $1 AND Id ~* ai;
-	IF FOUND THEN
-		IF r.InHouse = $3 THEN
-			RETURN TRUE;
-		END IF;
-	END IF;
-	RETURN FALSE;
-END;
-$$ LANGUAGE 'plpgsql' STABLE;
-
-CREATE OR REPLACE FUNCTION IfGroup() RETURNS Void AS $$
-DECLARE
-    r RECORD;
-BEGIN
-    SELECT INTO r Rolname FROM Pg_Authid WHERE Rolname = 'skyusers';
-    IF NOT FOUND THEN
-      CREATE GROUP SkyUsers;
-    END IF;
-END;
-$$ LANGUAGE 'plpgsql';
-
-SELECT InitSkyDb();
-
-CREATE OR REPLACE FUNCTION InitTables() RETURNS Void AS $$
+DO
+$$
 DECLARE
 	Pa VARCHAR;
 	I Integer;
@@ -965,25 +973,28 @@ BEGIN
 	 		 RAISE WARNING 'Initialize Stars';
 	 		 EXECUTE 'COPY Stars FROM ''' || Pa || '/Stars.txt'' DELIMITER AS ''|''';
 		END IF;
-END;
-$$ LANGUAGE 'plpgsql';
+END$$;
 
-SELECT InitTables();
-DROP FUNCTION InitTables();
 DROP VIEW Path;
 
 CREATE OR REPLACE FUNCTION GetInter(c VARCHAR, l CHAR(2)) RETURNS VARCHAR AS $$
 			 SELECT COALESCE((SELECT Text FROM Interpret WHERE Lang = l AND Code = c), (SELECT Text FROM Interpret WHERE Lang = 'en' AND Code = c));
 $$ LANGUAGE SQL;
+
 GRANT SELECT, INSERT, UPDATE, DELETE ON Data, Place,  Country, ObjRest, AspRest, Restrictions, VData, VDV, Obj, Asp, AspTbl, Ranks, Charts, Ring, Spreads TO GROUP SkyUsers;
+
 GRANT SELECT, UPDATE, USAGE ON Data_Idx_Seq, Place_Idx_Seq, Country_Idx_Seq, Restrictions_Idx_Seq, AspTbl_Idx_Seq, Chart_Idx_Seq TO GROUP SkyUsers;
+
 GRANT SELECT ON TypeIcons, TypeIcons_Idx_Seq, Degrees, ExtraObjs, D1901, Tarots, Parts, Interpret, Intertarots, ConsNames, Boundaries, Stars TO GROUP SkyUsers;
+
 GRANT EXECUTE ON FUNCTION IfSign(Real, Integer), TObj(Int4, VARCHAR, Integer), IfAsp(VARCHAR, VARCHAR, VARCHAR), TAsp(Int4, VARCHAR, VARCHAR, VARCHAR), TRetr(Int4, VARCHAR), THouse(Int4, VARCHAR, Integer), GetInter(c VARCHAR, l CHAR(2)), aExtract(CString, aDate) TO GROUP SkyUsers;
-DROP FUNCTION InitSkyDb();
+
 DROP FUNCTION IsField(VARCHAR, VARCHAR);
 DROP FUNCTION IsTable(VARCHAR);
 DROP FUNCTION IsFunction(VARCHAR);
 DROP FUNCTION IsConstraint(VARCHAR);
-CREATE OR REPLACE VIEW Version AS SELECT 4.2 AS V;
+
+CREATE OR REPLACE VIEW Version AS SELECT 4.21 AS V;
 GRANT SELECT ON Version TO GROUP SkyUsers;
+
 ANALYZE;
